@@ -14,25 +14,29 @@ module Network.DNS.Resolver (
   , lookupAuth
   -- ** Raw looking up function
   , lookupRaw
-  , lookupRaw'
+  , fromDNSMessage
   , fromDNSFormat
   ) where
 
-import Control.Applicative ((<$>), (<*>), pure)
 import Control.Exception (bracket)
-import qualified Data.ByteString.Char8 as BS
 import Data.Char (isSpace)
 import Data.List (isPrefixOf)
+import Data.Maybe (fromMaybe)
 import Network.BSD (getProtocolNumber)
 import Network.DNS.Decode
 import Network.DNS.Encode
 import Network.DNS.Internal
+import qualified Data.ByteString.Char8 as BS
 import Network.Socket (HostName, Socket, SocketType(Datagram), sClose, socket, connect)
-import Network.Socket (AddrInfoFlag(..), AddrInfo(..), defaultHints, getAddrInfo)
+import Network.Socket (AddrInfoFlag(..), AddrInfo(..), SockAddr(..), PortNumber(..), defaultHints, getAddrInfo)
 import Prelude hiding (lookup)
 import System.Random (getStdRandom, randomR)
 import System.Timeout (timeout)
 
+#if __GLASGOW_HASKELL__ < 709
+import Control.Applicative ((<$>), (<*>), pure)
+#endif
+
 #if mingw32_HOST_OS == 1
 import Network.Socket (send)
 import qualified Data.ByteString.Lazy.Char8 as LB
@@ -55,6 +59,7 @@ import Network.Socket.ByteString.Lazy (sendAll)
 --
 data FileOrNumericHost = RCFilePath FilePath -- ^ A path for \"resolv.conf\"
                        | RCHostName HostName -- ^ A numeric IP address
+                       | RCHostPort HostName PortNumber -- ^ A numeric IP address and port number
 
 -- | Type for resolver configuration. The easiest way to construct a
 --   @ResolvConf@ object is to modify the 'defaultResolvConf'.
@@ -129,14 +134,15 @@ makeResolvSeed conf = ResolvSeed <$> addr
                                  <*> pure (resolvBufsize conf)
   where
     addr = case resolvInfo conf of
-        RCHostName numhost -> makeAddrInfo numhost
-        RCFilePath file -> toAddr <$> readFile file >>= makeAddrInfo
+        RCHostName numhost -> makeAddrInfo numhost Nothing
+        RCHostPort numhost mport -> makeAddrInfo numhost $ Just mport
+        RCFilePath file -> toAddr <$> readFile file >>= \i -> makeAddrInfo i Nothing
     toAddr cs = let l:_ = filter ("nameserver" `isPrefixOf`) $ lines cs
                 in extract l
     extract = reverse . dropWhile isSpace . reverse . dropWhile isSpace . drop 11
 
-makeAddrInfo :: HostName -> IO AddrInfo
-makeAddrInfo addr = do
+makeAddrInfo :: HostName -> Maybe PortNumber -> IO AddrInfo
+makeAddrInfo addr mport = do
     proto <- getProtocolNumber "udp"
     let hints = defaultHints {
             addrFlags = [AI_ADDRCONFIG, AI_NUMERICHOST, AI_PASSIVE]
@@ -144,13 +150,21 @@ makeAddrInfo addr = do
           , addrProtocol = proto
           }
     a:_ <- getAddrInfo (Just hints) (Just addr) (Just "domain")
-    return a
+    let connectPort = case addrAddress a of
+                        SockAddrInet pn ha -> SockAddrInet (fromMaybe pn mport) ha
+                        SockAddrInet6 pn fi ha sid -> SockAddrInet6 (fromMaybe pn mport) fi ha sid
+                        unix -> unix
+    return $ a { addrAddress = connectPort }
 
 ----------------------------------------------------------------
 
 
 -- | Giving a thread-safe 'Resolver' to the function of the second
 --   argument. A socket for UDP is opened inside and is surely closed.
+--   Multiple 'withResolver's can be used concurrently.
+--   Multiple lookups must be done sequentially with a given
+--   'Resolver'. If multiple 'Resolver's are necessary for
+--   concurrent purpose, use 'withResolvers'.
 withResolver :: ResolvSeed -> (Resolver -> IO a) -> IO a
 withResolver seed func = bracket (openSocket seed) sClose $ \sock -> do
     connectSocket sock seed
@@ -158,6 +172,8 @@ withResolver seed func = bracket (openSocket seed) sClose $ \sock -> do
 
 -- | Giving thread-safe 'Resolver's to the function of the second
 --   argument. Sockets for UDP are opened inside and are surely closed.
+--   For each 'Resolver', multiple lookups must be done sequentially.
+--   'Resolver's can be used concurrently.
 withResolvers :: [ResolvSeed] -> ([Resolver] -> IO a) -> IO a
 withResolvers seeds func = bracket openSockets closeSockets $ \socks -> do
     mapM_ (uncurry connectSocket) $ zip socks seeds
@@ -192,40 +208,45 @@ getRandom = getStdRandom (randomR (0,65535))
 ----------------------------------------------------------------
 
 -- | Looking up resource records of a domain. The first parameter is one of
---   the field accessors of the 'DNSFormat' type -- this allows you to
+--   the field accessors of the 'DNSMessage' type -- this allows you to
 --   choose which section (answer, authority, or additional) you would like
 --   to inspect for the result.
 
-lookupSection :: (DNSFormat -> [ResourceRecord])
+lookupSection :: (DNSMessage -> [ResourceRecord])
               -> Resolver
               -> Domain
               -> TYPE
-              -> IO (Either DNSError [RDATA])
+              -> IO (Either DNSError [RData])
 lookupSection section rlv dom typ = do
     eans <- lookupRaw rlv dom typ
     case eans of
         Left  err -> return $ Left err
-        Right ans -> return $ fromDNSFormat ans toRDATA
+        Right ans -> return $ fromDNSMessage ans toRData
   where
     {- CNAME hack
     dom' = if "." `isSuffixOf` dom then dom else dom ++ "."
     correct r = rrname r == dom' && rrtype r == typ
     -}
     correct r = rrtype r == typ
-    toRDATA = map rdata . filter correct . section
+    toRData = map rdata . filter correct . section
 
--- | Extract necessary information from 'DNSFormat'
-fromDNSFormat :: DNSFormat -> (DNSFormat -> a) -> Either DNSError a
-fromDNSFormat ans conv = case errcode ans of
+-- | Extract necessary information from 'DNSMessage'
+fromDNSMessage :: DNSMessage -> (DNSMessage -> a) -> Either DNSError a
+fromDNSMessage ans conv = case errcode ans of
     NoErr     -> Right $ conv ans
     FormatErr -> Left FormatError
     ServFail  -> Left ServerFailure
     NameErr   -> Left NameError
     NotImpl   -> Left NotImplemented
     Refused   -> Left OperationRefused
+    BadOpt    -> Left BadOptRecord
   where
     errcode = rcode . flags . header
 
+-- | For backward compatibility.
+fromDNSFormat :: DNSMessage -> (DNSMessage -> a) -> Either DNSError a
+fromDNSFormat = fromDNSMessage
+
 -- | Look up resource records for a domain, collecting the results
 --   from the ANSWER section of the response.
 --
@@ -234,14 +255,14 @@ fromDNSFormat ans conv = case errcode ans of
 --   >>> let hostname = Data.ByteString.Char8.pack "www.example.com"
 --   >>> rs <- makeResolvSeed defaultResolvConf
 --   >>> withResolver rs $ \resolver -> lookup resolver hostname A
---   Right [93.184.216.119]
+--   Right [93.184.216.34]
 --
-lookup :: Resolver -> Domain -> TYPE -> IO (Either DNSError [RDATA])
+lookup :: Resolver -> Domain -> TYPE -> IO (Either DNSError [RData])
 lookup = lookupSection answer
 
 -- | Look up resource records for a domain, collecting the results
 --   from the AUTHORITY section of the response.
-lookupAuth :: Resolver -> Domain -> TYPE -> IO (Either DNSError [RDATA])
+lookupAuth :: Resolver -> Domain -> TYPE -> IO (Either DNSError [RData])
 lookupAuth = lookupSection authority
 
 
@@ -260,7 +281,7 @@ lookupAuth = lookupSection authority
 --   And the (formatted) expected output:
 --
 --   @
---   Right (DNSFormat
+--   Right (DNSMessage
 --           { header = DNSHeader
 --                        { identifier = 1,
 --                          flags = DNSFlags
@@ -271,10 +292,7 @@ lookupAuth = lookupSection authority
 --                                      recDesired = True,
 --                                      recAvailable = True,
 --                                      rcode = NoErr },
---                          qdCount = 1,
---                          anCount = 1,
---                          nsCount = 0,
---                          arCount = 0},
+--                        },
 --             question = [Question { qname = \"www.example.com.\",
 --                                    qtype = A}],
 --             answer = [ResourceRecord {rrname = \"www.example.com.\",
@@ -286,20 +304,15 @@ lookupAuth = lookupSection authority
 --             additional = []})
 --  @
 --
-lookupRaw :: Resolver -> Domain -> TYPE -> IO (Either DNSError DNSFormat)
+lookupRaw :: Resolver -> Domain -> TYPE -> IO (Either DNSError DNSMessage)
 lookupRaw = lookupRawInternal receive
 
--- | Like 'lookupRaw' except that no unknown RDATA records are not split up
---   into 'Int's.
-lookupRaw' :: Resolver -> Domain -> TYPE -> IO (Either DNSError (DNSMessage (RD BS.ByteString)))
-lookupRaw' = lookupRawInternal receive'
-
 lookupRawInternal ::
-    (Socket -> IO (DNSMessage a))
+    (Socket -> IO DNSMessage)
     -> Resolver
     -> Domain
     -> TYPE
-    -> IO (Either DNSError (DNSMessage a))
+    -> IO (Either DNSError DNSMessage)
 lookupRawInternal _ _   dom _
   | isIllegal dom     = return $ Left IllegalDomain
 lookupRawInternal rcv rlv dom typ = do
